Imports CrustCrawler.BuisnessLogicLayer

Public Class ChannelPropertySheet

    Private WithEvents ICh As IChannel
    Private prChannel As PeerChannel
    Private Channel As PscChannel
    Private CancelUpdateEvent As Boolean = False

    Public Event PropertyChanged As EventHandler


    Public Sub New(ByVal DataSource As Object)
        InitializeComponent()

        CancelUpdateEvent = True
        Dim DataType As Type = DataSource.GetType
        Select Case DataType.Name
            Case "PscChannel"
                Channel = CType(DataSource, PscChannel)
                prChannel = Nothing
                ICh = Channel
                DisableChildUI()
            Case "PeerChannel"
                prChannel = CType(DataSource, PeerChannel)
                ICh = prChannel
                Channel = Nothing
                BindChildCombo(prChannel)
        End Select
        CancelUpdateEvent = False

        Me.chkActive.Checked = True
        BindData()
    End Sub

    Private Sub DisableChildUI()
        Me.gbChildChannels.Enabled = False
    End Sub


    Private Sub BindData()
        Me.lblChannel.Text = "Channel " & CStr(ICh.ChannelId)
        Me.txtMax.Text = ICh.Max
        Me.txtPosition.Text = ICh.Position
        Me.txtMin.Text = ICh.Min
        Me.txtRamp.Text = ICh.Ramp
        Me.chkInverse.Checked = ICh.Inverse

        Me.Slider.Maximum = ICh.Max
        Me.Slider.Minimum = ICh.Min
        Me.Slider.TickFrequency = ICh.Max - ICh.Min / 3
        Me.Slider.Value = ICh.Position
        Me.lblInfo.Text = ""
    End Sub

    Private Sub BindChildCombo(ByVal channel As PeerChannel)
        Dim combo As ComboBox = Me.cboChannels
        For Each ch As PeerChannel In channel.ChildChannels
            combo.Items.Add("Channel" & ch.ChannelId)
            'Add child handlers
            AddHandler ch.PositionChanged, AddressOf ChildPositionChangedHandler
            AddHandler ch.OffsetChanged, AddressOf OffsetChangedHandler
        Next
        combo.SelectedIndex = 0
    End Sub


    Private Sub BindChildData(ByVal channel As PeerChannel)
        Dim childChannel As PeerChannel = GetChildDataSource(channel)

        Me.txtChildMax.Text = CStr(childChannel.Max)
        Me.lblChildPosition.Text = CStr(childChannel.position)
        Me.txtChildOffset.Text = CStr(childChannel.Offset)
        Me.txtChildMin.Text = CStr(childChannel.Min)
        Me.txtChildRamp.Text = CStr(childChannel.Ramp)
        Me.chkChildInverse.Checked = CStr(childChannel.Inverse)

        Me.ChildSlider.Maximum = childChannel.Max
        Me.ChildSlider.Minimum = childChannel.Min
        Me.ChildSlider.TickFrequency = childChannel.Max - childChannel.Min / 3
        Me.ChildSlider.Value = childChannel.position
    End Sub


    Private Function GetChildDataSource(ByVal channel As PeerChannel) As PeerChannel
        Return channel.ChildChannels(Me.cboChannels.SelectedIndex)
    End Function


    Private Sub Slider_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Slider.Scroll
        ICh.Position = CInt(CType(sender, TrackBar).Value)
        Me.txtPosition.Text = CStr(ICh.Position)
        Me.txtPosition.Update()
        Me.lblChildPosition.Update()
        If Me.chkActive.Checked Then
            ICh.move()
        End If
    End Sub

    Private Sub ChildSlider_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ChildSlider.Scroll
        Dim newPos As Integer = CInt(CType(sender, TrackBar).Value)
        Dim childChannel As PeerChannel = GetChildDataSource(prChannel)
        childChannel.position = newPos
        If Me.chkChildActive.Checked Then
            childChannel.Move()
        End If
    End Sub


    Private Sub txtPosition_LostFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtPosition.LostFocus
        Dim posTextbox = CType(sender, TextBox)
        Dim pos = CInt(posTextbox.text)
        If pos <= Me.Slider.Maximum And pos >= Me.Slider.Minimum Then
            Me.Slider.Value = pos
        Else
            Me.txtPosition.Undo()
            'posTextbox.text = 750
            Me.lblInfo.Text = String.Format("Position can not be >= {0} or <= {1}", Me.Slider.Maximum, Me.Slider.Minimum)
        End If
    End Sub


    Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click
        Me.Close()
    End Sub


    Private Sub btnApply_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnApply.Click
        Apply()
        'OnPropertyChanged(System.EventArgs.Empty)
    End Sub


    Private Sub Apply()
        ' Check for child channel updates
        If Not Me.cboChannels.SelectedIndex < 0 Then
            Dim childChannel As PeerChannel = GetChildDataSource(prChannel)
            childChannel.Inverse = Me.chkChildInverse.Checked
        End If
        ICh.Max = Me.txtMax.Text
        ICh.Position = Me.Slider.Value
        ICh.Min = Me.txtMin.Text
        ICh.Ramp = Me.txtRamp.Text
        ICh.Inverse = Me.chkInverse.Checked
        ICh.move()
    End Sub


    Private Sub btnOk_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOk.Click
        Apply()
        Me.Close()
        'OnPropertyChanged(System.EventArgs.Empty)
    End Sub


    'Private Sub OnPropertyChanged(ByVal e As System.EventArgs)
    '    RaiseEvent PropertyChanged(Me, e)
    'End Sub


#Region " UI "

    Private Sub txtMax_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtMax.GotFocus
        Me.lblInfo.Text = "Set the maximum position"
    End Sub

    Private Sub txtMin_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtMin.GotFocus
        Me.lblInfo.Text = "Set the minimum position"
    End Sub

    Private Sub txtPosition_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtPosition.GotFocus
        Me.lblInfo.Text = "Set position.  Use this setting to save a default servo position"
    End Sub

    Private Sub txtRamp_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtRamp.GotFocus
        Me.lblInfo.Text = "Set the rotation speed.  Ramp can have a value of 0-63, 0 is the quickest setting"
    End Sub

    Private Sub chkActive_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkActive.GotFocus
        Me.lblInfo.Text = "Activate the slider"
    End Sub

    Private Sub chkInverse_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkInverse.GotFocus
        Me.lblInfo.Text = "Inverse rotational direction"
    End Sub

    Private Sub ChannelPropertySheet_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Click
        Me.lblInfo.Text = ""
    End Sub

#End Region


    Private Sub cboChannels_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cboChannels.SelectedIndexChanged
        BindChildData(prChannel)
    End Sub

    Private Sub ICh_ChannelChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.ChannelChanged
        If Not prChannel Is Nothing Then
            BindChildData(prChannel)
        End If
    End Sub

    Private Sub ICh_MaxChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.MaxChanged
        Dim max As Integer = CType(sender, IChannel).Max
        If Me.Slider.Value > max Then
            Me.Slider.Maximum = max
            Me.Slider.Value = max
        Else
            Me.Slider.Maximum = max
        End If
    End Sub

    Private Sub ICh_MinChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.MinChanged
        Dim min As Integer = CType(sender, IChannel).Min
        If Me.Slider.Value < min Then
            Me.Slider.Value = min
            Me.Slider.Minimum = min
        Else
            Me.Slider.Minimum = min
        End If
    End Sub


    Private Sub ICh_PositionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.PositionChanged
        Dim pos As Integer

        If ICh.Inverse Then
            pos = Globals.Inverse(ICh.Max, ICh.Min, ICh.Position)
        Else
            pos = ICh.Position
        End If

        Me.txtPosition.Text = CStr(pos)
        Me.txtPosition.Update()
        Me.Slider.Value = pos
        Me.txtChildOffset.Update()
    End Sub


    Private Sub ICh_RampChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.RampChanged
        Me.txtRamp.Text = CType(sender, IChannel).Ramp
    End Sub


    Private Sub txtChildOffset_LostFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtChildOffset.LostFocus
        Dim newOffset As Integer = CInt(CType(sender, TextBox).Text)
        Dim childChannel As PeerChannel = GetChildDataSource(prChannel)
        If newOffset <> childChannel.Offset Then
            Dim diff = newOffset - childChannel.Offset
            childChannel.position += diff
        End If
        childChannel.Offset = newOffset
    End Sub

    'child Events
    Private Sub ChildPositionChangedHandler(ByVal sender As Object, ByVal e As EventArgs)
        Dim childChannel As PeerChannel = CType(sender, PeerChannel)
        'Me.ChildSlider.Value = CType(sender, PeerChannel).position
        'Me.txtChildOffset.Text = Me.ChildSlider.Value
        'Me.txtChildOffset.Update()
        Me.lblChildPosition.Text = childChannel.position
        Me.lblChildPosition.Update()
    End Sub

    Private Sub OffsetChangedHandler(ByVal sender As Object, ByVal e As EventArgs)
        Dim childChannel As PeerChannel = CType(sender, PeerChannel)

        Me.ChildSlider.Maximum = childChannel.Max
        Me.ChildSlider.Minimum = childChannel.Min
        Me.ChildSlider.Value = childChannel.position
        Me.ChildSlider.Update()
        Me.txtChildOffset.Text = CStr(childChannel.Offset)
        Me.txtChildOffset.Update()
        Me.txtChildMax.Text = CStr(childChannel.Max)
        Me.txtChildMin.Text = CStr(childChannel.Min)
        Me.lblChildPosition.Text = CStr(childChannel.position)



    End Sub

End Class