Imports System.IO
Imports System.Text
Imports System.Threading


Namespace BuisnessLogicLayer

    Public Class ParallaxServoController

        Private Const N2400 As String = "!SCSBR0" & vbCr
        Private Const N38k4 As String = "!SCSBR1" & vbCr
        Private Const VER As String = "!SCVER?" & vbCr
        Private Const RSP As String = "!SCRSP"
        Private Const HEADER As String = "!SC"

        Private _portName As String
        Private _baud As Integer
        Private _version As String
        Private _networked As Boolean = False '16 (default) or 32 channels
        Private _channels As New List(Of PscChannel)
        Private _id As Integer


        Private connection As New IO.Ports.SerialPort

#Region " Enums "
        Public Enum BaudRate As Integer
            N2400 = 2400
            N38k4 = 38400
        End Enum

        Public Enum NumberOfChannels As Byte
            NotNetworked = 16
            Networked = 32
        End Enum
#End Region

#Region " Constructor "

        Public Sub New()
        End Sub

        Public Sub New(ByVal portName As String, ByVal baud As Integer, ByVal version As String)
            _portName = portName
            _baud = baud
            _version = version
        End Sub 'New

#End Region

#Region "Properties"

        Public ReadOnly Property PortName() As String
            Get
                Return _portName
            End Get
        End Property 'PortName


        Public Property Baud() As BaudRate
            Get
                Return _baud
            End Get
            Set(ByVal value As BaudRate)
                _baud = value
            End Set
        End Property 'BaudRate


        Public ReadOnly Property Version() As String
            Get
                Return _version
            End Get

        End Property 'Version


        Public Property Networked() As Boolean
            Get
                Return _networked
            End Get
            Set(ByVal value As Boolean)
                _networked = value
            End Set
        End Property 'Networked


        Public Property Channel(ByVal index As Integer) As PscChannel
            Get
                Return _channels(index)
            End Get
            Set(ByVal value As PscChannel)
                _channels(index) = value
            End Set
        End Property 'Channel


        Public Property Channel() As List(Of PscChannel)
            Get
                Return _channels
            End Get
            Set(ByVal value As List(Of PscChannel))
                _channels = value
            End Set
        End Property 'Channel


        Public Property Id() As Integer
            Get
                Return _id
            End Get
            Set(ByVal value As Integer)
                _id = value
            End Set
        End Property


#End Region


        Public Function CreateNewChannel(ByVal ChannelId As Integer) As PscChannel
            Dim ch As New PscChannel(ChannelId, Me.Id)
            AddHandler ch.PositionCommand, AddressOf PositonCommandHandler
            _channels.Add(ch)
            Return ch
        End Function 'CreateNewChannel

        Public Function CreateNewChannel(ByVal channel As PscChannel)
            Dim ch As New PscChannel(channel.ChannelId, channel.PscId, channel.Max, channel.Min, channel.position, channel.Ramp, channel.Inverse, channel.Delay, channel.Grouped)
            AddHandler ch.PositionCommand, AddressOf PositonCommandHandler
            _channels.Add(ch)
            Return ch
        End Function

        Public Sub RemoveChannel(ByVal ch As PscChannel)
            _channels.Remove(ch)
        End Sub 'RemoveChannel


        Public Function CreatePeerGroup(ByVal Master As PscChannel, ByVal Child As PscChannel) As PeerChannel
            Dim prChannel As PeerChannel = PeerChannel.CreateNewGroup(Master, Child)
            AddHandler prChannel.PositionCommand, AddressOf PositonCommandHandler
            AddHandler prChannel.ChildChannel(0).PositionCommand, AddressOf PositonCommandHandler
            Return prChannel
        End Function 'CreatePeerGroup


        Public Function AddPeerChild(ByVal Group As PeerChannel, ByVal NewChild As PscChannel) As PeerChannel
            Dim index As Integer = PeerChannel.AddPeerChild(Group, NewChild)
            AddHandler Group.ChildChannel(index).PositionCommand, AddressOf PositonCommandHandler
            Return Group
        End Function 'AddPeerChild


        Public Overloads Sub OpenSerialPort()
            connection = My.Computer.Ports.OpenSerialPort(Me.PortName, Me.Baud)
        End Sub 'OpenSerialPort


        Private Overloads Sub OpenSerialPort(ByVal baud As BaudRate)
            connection = My.Computer.Ports.OpenSerialPort(Me.PortName, baud)
        End Sub 'OpenSerialPort

        Private Overloads Sub OpenSerialPort(ByVal portName As String, ByVal baud As BaudRate)
            connection = My.Computer.Ports.OpenSerialPort(portName, baud)
        End Sub 'OpenSerialPort


        Public Sub CloseSerialPort()
            connection.Close()
        End Sub 'CloseSerialPort


        Public Function IsSerialPortOpen() As Boolean
            If connection.IsOpen Then Return True
            Return False
        End Function 'IsSerialPortOpen


        Public Function Ping() As Boolean
            If IsSerialPortOpen() Then
                Dim buffer(11) As Char

                connection.WriteLine(VER)
                Thread.Sleep(200)
                connection.Read(buffer, 0, 12)

                If Not buffer(9) = Nothing Then Return True
            Else
                Throw New IOException("Ping error -- serial port: " & Me.PortName & " is closed")
            End If
            Return False
        End Function 'Ping


        Public Function SetBaud(ByVal baud As BaudRate) As BaudRate
            If IsSerialPortOpen() AndAlso Not Me.Baud = baud Then
                Select Case baud
                    Case BaudRate.N2400
                        connection.WriteLine(N2400)
                        CloseSerialPort()
                        OpenSerialPort(BaudRate.N2400)
                        Me.Baud = BaudRate.N2400
                    Case BaudRate.N38k4
                        connection.WriteLine(N38k4)
                        CloseSerialPort()
                        OpenSerialPort(BaudRate.N38k4)
                        Me.Baud = BaudRate.N38k4
                End Select
            End If
            Return baud
        End Function 'SetBaud


        Public Function ReportServoPosition(ByVal channel As Integer) As Integer
            If Me.IsSerialPortOpen Then
                Dim cmd As String = RSP & channel & Chr(13)
                Dim buffer(2) As Byte

                connection.ReadTimeout = 500
                connection.WriteTimeout = 500

                Try
                    connection.WriteLine(cmd)
                    Thread.Sleep(200)
                    Dim reply As String = connection.ReadLine()
                    If Mid(reply, 1, RSP.Length - 1) = RSP Then
                        Return 0
                    Else
                        connection.Read(buffer, 0, 3)
                        Return Dpos(buffer(1), buffer(2))
                    End If

                Catch ex As TimeoutException
                    MessageBox.Show(ex.ToString)
                Catch ex As Exception
                    MessageBox.Show(ex.ToString)
                End Try

            End If
        End Function 'ReportServoPosition

        Private Function Dpos(ByVal highbyte As Byte, ByVal lowbyte As Byte) As Integer
            Try
                Return ((Hex(highbyte) << 8) + lowbyte)
            Catch ex As Exception
                Return 0
                'MessageBox.Show(ex.ToString)
            End Try
        End Function 'Dpos


        Public Sub PositonCommandHandler(ByVal sender As Object, ByVal e As PscEventArgs)
            If Me.IsSerialPortOpen Then
                Dim data As Byte() = FormatPscPositionCommand(e.Id, e.Ramp, e.Position)
                Thread.Sleep(DefaultValues.Delay)
                connection.Write(HEADER)
                connection.Write(data, 0, 5)
            End If
        End Sub 'PositonCommandHandler


        Public Shared Function FormatPscPositionCommand(ByVal channel As Integer, ByVal ramp As Integer, ByVal postion As Integer) As Byte()
            Dim data(4) As Byte
            data(0) = CByte(channel)
            data(1) = CByte(ramp)
            data(2) = CByte((postion And &HFF))
            data(3) = CByte((postion And &HFF00) / &H100)
            data(4) = &HD
            Return data
        End Function 'FormatPscPositionCommand

        'Overrides
        Public Overrides Function ToString() As String
            Return String.Format("| PSC({0}) {1} {2} {3} |", Me.Id, Me.PortName, Me.Baud, Me.Version)
        End Function 'ToString

    End Class 'ParallaxServoController

End Namespace 'BuisnessLogicLayer

