Imports System.IO
Imports System.Threading


Namespace BuisnessLogicLayer

  Public Class ParallaxServoControllerFactory
    'Members
    Private Const VER As String = "!SCVER?" & vbCr
    Private Const N2400 As Integer = 2400
    Private Const N38k4 As Integer = 38400

    Private pscGenericList As New List(Of ParallaxServoController)

    'Default constructor
    Public Sub New()
    End Sub


    Public Shared Function GetPSCs() As List(Of ParallaxServoController)
      Dim pscFactory As New ParallaxServoControllerFactory
      Try
        Dim serialPortNames As List(Of String) = pscFactory.GetSerialPortNames()
        Return pscFactory.GetParallaxServoControllers(serialPortNames)
      Catch ex As Exception
        MessageBox.Show(ex.ToString)
        Return Nothing
      End Try
    End Function


    'Return system serial ports
    Public Function GetSerialPortNames() As List(Of String)
      Dim SerialPortNames As New List(Of String)
      For Each sp As String In My.Computer.Ports.SerialPortNames
        SerialPortNames.Add(sp)
      Next
      Return SerialPortNames
    End Function


    'Loop through each serial port in the system at 2400 then 38k4
    Private Function GetParallaxServoControllers(ByVal SerialPortNames As List(Of String)) As List(Of ParallaxServoController)
      Dim version As String
      Dim portName As String = 0
      Try
        For Each portName In SerialPortNames
          Using connection As IO.Ports.SerialPort = My.Computer.Ports.OpenSerialPort(portName, N2400)
            version = PingForPSC(connection)
            If Not version Is Nothing Then
              Dim psc As New ParallaxServoController(portName, N2400, version)
              pscGenericList.Add(psc)
              Dim idx As Integer = pscGenericList.IndexOf(psc)
              pscGenericList.Item(idx).id = idx
            End If
          End Using

          Using connection As IO.Ports.SerialPort = My.Computer.Ports.OpenSerialPort(portName, N38k4)
            version = PingForPSC(connection)
            If Not version Is Nothing Then
              Dim psc As New ParallaxServoController(portName, N38k4, version)
              pscGenericList.Add(psc)
              Dim idx As Integer = pscGenericList.IndexOf(psc)
              pscGenericList.Item(idx).Id = idx
            End If
          End Using
        Next

      Catch ex As UnauthorizedAccessException
        'MessageBox.Show(ex.ToString)
        Return Nothing
      Catch ex As Exception
        'MessageBox.Show(ex.ToString)
        Return Nothing
      End Try

      Return pscGenericList
    End Function


    'Send PSC Ver command
    Private Function PingForPSC(ByVal connection As IO.Ports.SerialPort) As String
      Dim buffer(11) As Char
      connection.ReadTimeout = 500
      connection.WriteTimeout = 500
      Try
        connection.WriteLine(VER)
        Thread.Sleep(200)
        connection.Read(buffer, 0, 12)
        connection.DiscardOutBuffer()
        connection.DiscardInBuffer()

      Catch ex As TimeoutException
        Return Nothing
      Catch ex As Exception
        MessageBox.Show(ex.ToString)
      End Try

      If buffer(9) = Nothing Then
        Return Nothing
      Else
        Return buffer(9) & buffer(10) & buffer(11)
      End If
    End Function

  End Class


End Namespace

