Imports System.IO
Imports System.Text
Imports System.Threading

Namespace BuisnessLogicLayer


  Public Class PscChannel
        Implements IChannel


        Protected _channelId As Integer
        Protected _max As Integer
        Protected _min As Integer
        Protected _position As Integer
        Protected _ramp As Integer
        Protected _inverse As Boolean = False
        Protected _delay As Integer
        Protected _grouped As Boolean
        Protected _pscId As Integer


        'Interface events
        Public Event PositionCommand As PositionCommandEventHandler Implements IChannel.PositionCommand
        Public Event PositionChanged As EventHandler Implements IChannel.PositionChanged
        Public Event RampChanged As EventHandler Implements IChannel.RampChanged
        Public Event MaxChanged As EventHandler Implements IChannel.MaxChanged
        Public Event MinChanged As EventHandler Implements IChannel.MinChanged
        Public Event DelayChanged As EventHandler Implements IChannel.DelayChanged
        Public Event ChannelChanged As EventHandler Implements IChannel.ChannelChanged
        Public Event InverseChanged As EventHandler Implements IChannel.InverseChanged



#Region "Contructors"
        Public Sub New()
            MyClass.New(DefaultValues.ChannelId, DefaultValues.ChannelId, DefaultValues.Max, DefaultValues.Min, DefaultValues.Position, DefaultValues.Ramp, DefaultValues.Inverse, DefaultValues.Delay, DefaultValues.Grouped)
        End Sub

        Public Sub New(ByVal Id As Integer, ByVal PscId As Integer)
            MyClass.New(Id, PscId, DefaultValues.Max, DefaultValues.Min, DefaultValues.Position, DefaultValues.Ramp, DefaultValues.Inverse, DefaultValues.Delay, DefaultValues.Grouped)
        End Sub

        Public Sub New(ByVal channelId As Integer, ByVal pscId As Integer, ByVal max As Integer, ByVal min As Integer, ByVal position As Integer, ByVal ramp As Integer, ByVal inverse As Boolean, ByVal delay As Integer, ByVal grouped As Boolean)
            _channelId = channelId
            _pscId = pscId
            _max = max
            _min = min
            _position = position
            _ramp = ramp
            _inverse = inverse
            _delay = delay
            _grouped = grouped
        End Sub
#End Region


#Region "Properties"

        Public ReadOnly Property ChannelId() As Integer Implements IChannel.ChannelId
            Get
                Return _channelId
            End Get
        End Property


        Public Property Max() As Integer Implements IChannel.Max
            Get
                Return _max
            End Get
            Set(ByVal value As Integer)
                If _max <> value Then
                    _max = value
                    OnMaxChanged(EventArgs.Empty)
                End If
            End Set
        End Property


        Public Property Min() As Integer Implements IChannel.Min
            Get
                Return _min
            End Get
            Set(ByVal value As Integer)
                If _min <> value Then
                    _min = value
                    OnMinChanged(EventArgs.Empty)
                End If
            End Set
        End Property


        Public Overridable ReadOnly Property FindCenterPoint() As Integer
            Get
                Return (_max + _min) / 2
            End Get
        End Property


        Public Property position() As Integer Implements IChannel.Position
            Get
                If Me.Inverse Then
                    Return Globals.Inverse(_max, _min, _position)
                Else
                    Return _position
                End If
            End Get
            Set(ByVal value As Integer)
                Select Case value
                    Case Is < _min
                        _position = _min
                    Case Is > _max
                        _position = _max
                End Select
                If _position <> value Then
                    _position = value
                    OnPositionChanged(EventArgs.Empty)
                End If
            End Set
        End Property



        Public Property Ramp() As Integer Implements IChannel.Ramp
            Get
                Return _ramp
            End Get
            Set(ByVal value As Integer)
                Select Case value
                    Case Is < 0
                        _ramp = 0
                    Case Is > 63
                        _ramp = 63
                End Select
                If _ramp <> value Then
                    _ramp = value
                    OnRampChanged(EventArgs.Empty)
                End If
            End Set
        End Property


        Public Property Inverse() As Boolean Implements IChannel.Inverse
            Get
                Return _inverse
            End Get
            Set(ByVal value As Boolean)
                If _inverse <> value Then
                    _inverse = value
                    OnInverseChanged(System.EventArgs.Empty)
                End If

            End Set
        End Property

        Public Property Delay() As Integer Implements IChannel.Delay
            Get
                Return _delay
            End Get
            Set(ByVal value As Integer)
                If _delay <> value Then
                    If value < 256 Then
                        _delay = value
                    Else
                        _delay = 255
                    End If
                    OnDelayChanged(EventArgs.Empty)
                End If
            End Set
        End Property 'Delay


        Public Property Grouped() As Boolean
            Get
                Return _grouped
            End Get
            Set(ByVal value As Boolean)
                _grouped = value
            End Set
        End Property


        Public Property PscId() As Integer Implements IChannel.PscId
            Get
                Return _pscId
            End Get
            Set(ByVal value As Integer)
                _pscId = value
            End Set
        End Property

#End Region

        Public Overridable Sub move() Implements IChannel.move
            Dim e As New PscEventArgs(Me)
            OnMove(e)
        End Sub


        Public Sub OnMove(ByVal e As PscEventArgs)
            RaiseEvent PositionCommand(Me, e)
            RaiseEvent ChannelChanged(Me, EventArgs.Empty)
            Thread.Sleep(Me.Delay)
        End Sub

        Public Overridable Sub OnMaxChanged(ByVal e As EventArgs)
            RaiseEvent MaxChanged(Me, e)
            RaiseEvent ChannelChanged(Me, EventArgs.Empty)
        End Sub

        Public Overridable Sub OnMinChanged(ByVal e As EventArgs)
            RaiseEvent MinChanged(Me, e)
            RaiseEvent ChannelChanged(Me, EventArgs.Empty)
        End Sub

        Public Overridable Sub OnPositionChanged(ByVal e As EventArgs)
            RaiseEvent PositionChanged(Me, e)
            RaiseEvent ChannelChanged(Me, EventArgs.Empty)
        End Sub

        Public Overridable Sub OnRampChanged(ByVal e As EventArgs)
            RaiseEvent RampChanged(Me, e)
            RaiseEvent ChannelChanged(Me, EventArgs.Empty)
        End Sub

        Public Overridable Sub OnDelayChanged(ByVal e As EventArgs)
            RaiseEvent DelayChanged(Me, e)
            RaiseEvent ChannelChanged(Me, EventArgs.Empty)
        End Sub

        Public Overridable Sub OnInverseChanged(ByVal e As EventArgs)
            RaiseEvent InverseChanged(Me, e)
            RaiseEvent ChannelChanged(Me, EventArgs.Empty)
        End Sub


        Public Overrides Function ToString() As String
            Dim returnString As String
            If Me.ChannelId >= 0 Then
                returnString = "Channel:   " & Me.ChannelId & vbCrLf & _
                                "Max :     " & Me.Max & vbCrLf & _
                                "Min:      " & Me.Min & vbCrLf & _
                                "Position: " & Me.position & vbCrLf & _
                                "Ramp:     " & Me.Ramp & vbCrLf & _
                                "Inverse:  " & Me.Inverse & vbCrLf & _
                                "Delay:    " & Me.Delay
            Else
                Return "Invalid Channel Number"
            End If

            Return returnString
        End Function

        Public Shared Function Empty() As PscChannel
            Dim n As PscChannel = Nothing
            Return n
        End Function


    End Class

End Namespace