﻿Imports CrustCrawler.BuisnessLogicLayer
Imports System.Threading
Imports CrustCrawler.BuisnessLogicLayer.ParallaxServoController

Partial Class Project

    Public Event ProjectChanged As EventHandler
    Public Event LoadProject As EventHandler
    Public Event PscTableUpdate As EventHandler

    Private _changed As Boolean

    Public Property Changed() As Boolean
        Get
            Return _changed
        End Get
        Set(ByVal value As Boolean)
            _changed = value
        End Set
    End Property


    Public Sub OnUpdateProject(ByVal psc As List(Of ParallaxServoController), ByVal pscChannelContrls As List(Of PscChannelControl))
        Me.Clear()

        RefreshPscTable(psc)
        RefreshPscChannel(pscChannelContrls)
    End Sub

    Private Sub RefreshPscChannel(ByVal pscChannelContrls As List(Of PscChannelControl))
        Dim newRow As Project.PscChannelRow

        'Clear rows
        Me.PscChildChannel.Clear()
        Me.PscChannel.Rows.Clear()
        _changed = True

        For Each pcc As PscChannelControl In pscChannelContrls
            newRow = Me.PscChannel.NewPscChannelRow
            newRow.ChannelId = pcc.DataSource.ChannelId
            newRow.LocationX = pcc.Location.X
            newRow.LocationY = pcc.Location.Y
            newRow.PscId = pcc.DataSource.PscId
            newRow.Position = pcc.DataSource.position
            newRow.Ramp = pcc.DataSource.Ramp
            newRow.Max = pcc.DataSource.Max
            newRow.Min = pcc.DataSource.Min
            newRow.Inverse = pcc.DataSource.Inverse
            newRow.Delay = pcc.DataSource.Delay
            newRow.Grouped = pcc.DataSource.Grouped
            Me.PscChannel.Rows.Add(newRow)
            Dim identity As Integer = newRow.PscChannelId
            'Add Child objects if this is a grouped channel
            If pcc.DataSource.GetType.Name = "PeerChannel" Then
                RefreshChildChannel(pcc.DataSource, identity)
            End If
        Next
    End Sub


    Private Sub RefreshChildChannel(ByVal pChannel As PeerChannel, ByVal identity As Integer)
        Dim newRow As Project.PscChildChannelRow
        Dim children As List(Of PeerChannel) = pChannel.ChildChannels

        For Each p As PeerChannel In children
            newRow = Me.PscChildChannel.NewPscChildChannelRow
            newRow.ChannelId = p.ChannelId
            newRow.PscId = p.PscId
            newRow.Position = p.position
            newRow.Ramp = p.Ramp
            newRow.Max = p.Max
            newRow.Min = p.Min
            newRow.Delay = p.Delay
            newRow.Grouped = p.Grouped
            newRow.Inverse = p.Inverse
            newRow.PscChannelId = identity
            Me.PscChildChannel.Rows.Add(newRow)
        Next
    End Sub


    Private Sub RefreshPscTable(ByVal psc As List(Of ParallaxServoController))
        Dim newRow As Project.PscRow

        'Clear Rows
        Me.Psc.Rows.Clear()
        _changed = True

        For Each p As ParallaxServoController In psc
            newRow = Me.Psc.NewPscRow
            newRow.PscId = p.Id
            newRow.Baud = p.Baud
            newRow.Port = p.PortName
            Me.Psc.AddPscRow(newRow)
        Next
    End Sub


    Public Sub WriteToFile(ByVal fileLocation As String)
        Me.WriteXml(fileLocation, XmlWriteMode.WriteSchema)
    End Sub


    Public Sub LoadXmlData(ByVal fileLocation As String)
        If Me IsNot Nothing Then
            Me.Clear()
        End If
        Me.ReadXml(fileLocation, XmlReadMode.ReadSchema)
        OnLoadProject(EventArgs.Empty)
    End Sub


    Public Sub OnProjectChanged(ByVal e As EventArgs)
        RaiseEvent ProjectChanged(Me, e)
    End Sub

    Public Sub OnLoadProject(ByVal e As EventArgs)
        RaiseEvent LoadProject(Me, e)
    End Sub

End Class
