Imports CrustCrawler.BuisnessLogicLayer
Imports CrustCrawler.BuisnessLogicLayer.ParallaxServoController
Imports System.Reflection
Imports System.Drawing
Imports System.Collections.Generic

Public Class PscChannelControl : Inherits UserControl

    Public Event ViewLocationChange As EventHandler
    Public Event ControlMoved As EventHandler
    Public Event Group As GroupEventHandler
    Public Event UnGroup As GroupEventHandler
    Public Event StateChanged As EventHandler


    Private WithEvents ICh As IChannel
    Private Channel As PscChannel
    Private prChannel As PeerChannel

    Private _changed As Boolean = False
    Private CancelUpdateEvent As Boolean = False
    Private dragSlider As Boolean = False

    'Drag operation
    Private rec As Rectangle
    Private offsetInRecangle As Point
    Private dragBoxFromMouseDown As Rectangle
    Private mousePositionInControl As Point
    Private ControlDragOccured As Boolean = False





    Public Property Changed() As Boolean
        Get
            Return _changed
        End Get
        Set(ByVal value As Boolean)
            _changed = value
        End Set
    End Property


    Public ReadOnly Property DataSource() As PscChannel
        Get
            If Channel Is Nothing Then
                Return prChannel
            Else
                Return Channel
            End If
        End Get
    End Property

    Public ReadOnly Property ChannelId() As String
        Get
            Return ICh.ChannelId
        End Get
    End Property



    Public Sub BindChannel(ByVal DataSource As PscChannel)
        If DataSource Is Nothing Then
            Throw New ArgumentNullException("Assign a datasource before binding to " & Me.ToString)
        End If
        CancelUpdateEvent = True
        Select Case DataSource.GetType.Name
            Case "PscChannel"
                Channel = CType(DataSource, PscChannel)
                prChannel = Nothing
                ICh = Channel
            Case "PeerChannel"
                prChannel = CType(DataSource, PeerChannel)
                ICh = prChannel
                Channel = Nothing
        End Select
        tslChannelNum.Text = ICh.PscId & "/" & ICh.ChannelId.ToString
        UpdateUI()
        CancelUpdateEvent = False
    End Sub 'BindChannel


    Private Sub UpdateUI()
        Me.SetPosition(CStr(ICh.Position))
        Me.SetSlider(CStr(ICh.Position))
        Me.SetRamp(CStr(ICh.Ramp))
        Me.SetDelay(CStr(ICh.Delay))
    End Sub 'UpdateUI


    Private Sub Slider_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Slider.MouseDown
        dragSlider = True
        If e.Button = Windows.Forms.MouseButtons.Right Then
            Me.ContextMenuStrip1.Show(Me.PointToScreen(New Point(e.X, e.Y + 50)))
        End If
    End Sub


    Private Sub Slider_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Slider.MouseUp
        dragSlider = False
        If Not Me.chkActive.Checked Then
            ICh.move()
        End If
    End Sub


    Private Sub Slider_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Slider.Scroll
        ICh.Position = Me.Slider.Value
        If Me.chkActive.Checked Then
            ICh.move()
        End If
        Me.SetPosition(CStr(Me.Slider.Value))
    End Sub


    Private Sub ICh_ChannelChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.ChannelChanged
        _changed = True
    End Sub


    Private Sub ICh_MaxChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.MaxChanged
        Dim max As Integer = CType(sender, IChannel).Max
        If Me.Slider.Value > max Then
            Me.Slider.Maximum = max
            Me.Slider.Value = max
        Else
            Me.Slider.Maximum = max
        End If
    End Sub


    Private Sub GetTicks()
        Dim ticks As Integer = Me.Slider.Maximum - Me.Slider.Minimum / 2
        Me.Slider.TickFrequency = ticks
    End Sub


    Private Sub ICh_MinChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.MinChanged
        Dim min As Integer = CType(sender, IChannel).Min
        If Me.Slider.Value < min Then
            Me.Slider.Value = min
            Me.Slider.Minimum = min
        Else
            Me.Slider.Minimum = min
        End If
        'GetTicks()
    End Sub


    Private Sub ICh_PositionChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.PositionChanged
        Dim pos As Integer

        If ICh.Inverse Then
            pos = Globals.Inverse(ICh.Max, ICh.Min, ICh.Position)
        Else
            pos = ICh.Position
        End If

        Me.SetPosition(CStr(pos))
        'Me.txtPosition.Update()
        Me.SetSlider(CStr(pos))
    End Sub


    Delegate Sub SetTextCallback(ByVal [text] As String)

    'SetPosition allows for thread safe calls to windows forms objects
    Private Sub SetPosition(ByVal [text] As String)
        If Me.txtPosition.InvokeRequired Then
            Dim d As New SetTextCallback(AddressOf SetPosition)
            Me.Invoke(d, New Object() {[text]})
        Else
            'If Me.DataSource.Inverse Then
            '    [text] = Globals.Inverse(Me.DataSource.Max, Me.DataSource.Min, CInt([text]))
            'End If
            Me.txtPosition.Text = [text]
            Me.txtPosition.Update()
        End If
    End Sub

    'SetSlider allows for thread safe calls to windows forms objects
    Private Sub SetSlider(ByVal [text] As String)
        If Me.Slider.InvokeRequired Then
            Dim d As New SetTextCallback(AddressOf SetSlider)
            Me.Invoke(d, New Object() {[text]})
        Else
            'If Me.DataSource.Inverse Then
            '    [text] = Globals.Inverse(Me.DataSource.Max, Me.DataSource.Min, CInt([text]))
            'End If
            Me.Slider.Value = CInt([text])
            Me.Slider.Update()
        End If
    End Sub


    Private Sub IChnl_RampChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.RampChanged
        Dim newRamp As Integer = CType(sender, IChannel).Ramp
        Me.SetRamp(CStr(newRamp))
        'Me.txtRamp.Update()
    End Sub

    'SetRamp allows for thread safe calls to windows forms objects
    Private Sub SetRamp(ByVal [text] As String)
        If Me.txtRamp.InvokeRequired Then
            Dim d As New SetTextCallback(AddressOf SetRamp)
            Me.Invoke(d, New Object() {[text]})
        Else
            Me.txtRamp.Text = [text]
            Me.txtRamp.Update()
        End If
    End Sub


    Private Sub ICh_DelayChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.DelayChanged
        Dim delay As Integer = CType(sender, IChannel).Delay
        Me.SetDelay(CStr(delay))
        'Me.txtDelay.Update()
    End Sub

    'SetDelay allows for thread safe calls to windows forms objects
    Private Sub SetDelay(ByVal [text] As String)
        If Me.txtDelay.InvokeRequired Then
            Dim d As New SetTextCallback(AddressOf SetDelay)
            Me.Invoke(d, New Object() {[text]})
        Else
            Me.txtDelay.Text = [text]
            Me.txtDelay.Update()
        End If
    End Sub




    Private Sub ICh_PositionCommand(ByVal sender As Object, ByVal e As BuisnessLogicLayer.PscEventArgs) Handles ICh.PositionCommand
        Dim pos As Integer

        If ICh.Inverse Then
            pos = Globals.Inverse(ICh.Max, ICh.Min, ICh.Position)
        Else
            pos = ICh.Position
        End If

        Me.SetPosition(CStr(pos))
        'Me.txtPosition.Update()
        Me.SetSlider(CStr(pos))
    End Sub




    Private Sub txtPosition_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles txtPosition.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            'SendKeys.Send("{TAB}")
            Me.txtRamp.Focus()
            If Me._changed Then
                ICh.move()
            End If
        End If
    End Sub


    Private Sub PostionTextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtPosition.LostFocus
        ICh.Position = CInt(CType(sender, TextBox).Text)
    End Sub

    Private Sub txtRamp_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles txtRamp.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            Me.txtDelay.Focus()
            If Me._changed Then
                ICh.move()
            End If
        End If
    End Sub


    Private Sub RampChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtRamp.LostFocus
        ICh.Ramp = CInt(CType(sender, TextBox).Text)
    End Sub

    Private Sub txtDelay_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles txtDelay.KeyDown
        If e.KeyCode = Keys.Enter Then
            e.SuppressKeyPress = True
            Me.txtPosition.Focus()
            If Me._changed Then
                ICh.move()
            End If
        End If
    End Sub


    Private Sub DelayChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtDelay.LostFocus
        ICh.Delay = CInt(CType(sender, TextBox).Text)
    End Sub


    Private Sub tsbProperties_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbProperties.Click

        If Not Channel Is Nothing Then
            Dim propForm As New ChannelPropertySheet(Me.Channel)
            propForm.Show()
        End If

        If Not prChannel Is Nothing Then
            Dim propForm As New ChannelPropertySheet(Me.prChannel)
            propForm.Show()
        End If
    End Sub


    Public Overrides Function ToString() As String
        Return Me.Name
    End Function 'ToString


    'Graphics and Drag operations
    Private Sub PscChannelControl_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        Dim x As Integer = Me.Width
        Dim y As Integer = Me.Height
        'Left border
        e.Graphics.DrawLine(Pens.Black, 1, 25, 1, y - 2)
        'Bottom border
        e.Graphics.DrawLine(Pens.Black, 1, y - 2, x - 2, y - 2)
        'Right border
        e.Graphics.DrawLine(Pens.Black, x - 2, 25, x - 2, y - 2)

        UpdateGroupLabel()
    End Sub

    Private Sub UpdateGroupLabel()
        Dim str As New List(Of String)
        Dim strA As String = Nothing

        If Me.DataSource.GetType.Name = "PeerChannel" Then
            Dim children As PeerChannel = CType(Me.DataSource, PeerChannel)
            For Each c As PeerChannel In prChannel.ChildChannels
                str.Add(String.Format("{0}/{1}", c.PscId, c.ChannelId))
            Next
            str.Sort()
            For Each s As String In str
                strA = strA & s.ToString & ","
            Next
            Me.lblGroup.Text = Mid(strA, 1, strA.Length - 1)
        Else
            Me.lblGroup.Text = String.Empty
        End If
        Me.lblGroup.Update()
    End Sub




    Private Sub ToolStrip1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ToolStrip1.MouseDown
        Try
            Me.BringToFront()

            Dim mousePointToScreen As Point = Me.PointToScreen(New Point(e.X, e.Y))
            mousePositionInControl = New Point(e.X, e.Y)
            Dim controlLocation As Point = mousePointToScreen - mousePositionInControl

            'Another way to find the control location on the screen
            'Dim pscControlPoint As Point = Me.PointToScreen(Me.Location) - Me.Location
            'MessageBox.Show(pscControlPoint.ToString & vbCrLf & Me.PointToScreen(Me.Location).ToString & vbCrLf & Me.Location.ToString & vbCrLf & mousePosition.ToString & vbCrLf & e.Location.ToString)

            'rec = New Rectangle(controlLocation, Me.Size)
            'rec.Inflate(2, 2)
            'ControlPaint.DrawReversibleFrame(rec, SystemColors.Highlight, FrameStyle.Thick)

            ' Remember the point where the mouse down occurred. The DragSize indicates
            ' the size that the mouse can move before a drag event should be started.                
            Dim dragSize As Size = SystemInformation.DragSize

            ' Create a rectangle using the DragSize, with the mouse position being
            ' at the center of the rectangle.
            dragBoxFromMouseDown = New Rectangle(New Point(e.X - (dragSize.Width / 2), e.Y - (dragSize.Height / 2)), dragSize)
        Catch ex As Exception
            MessageBox.Show(ex.ToString)
        End Try

    End Sub


    Private Sub ToolStrip1_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ToolStrip1.MouseMove
        If ((e.Button And MouseButtons.Left) = MouseButtons.Left) Then
            ' If the mouse moves outside the rectangle, start the drag.
            If (Rectangle.op_Inequality(dragBoxFromMouseDown, Rectangle.Empty) And Not dragBoxFromMouseDown.Contains(e.X, e.Y)) Then
                Try
                    ControlDragOccured = True
                    OnViewLocationChange(e)
                    'Write over first rec
                    ControlPaint.DrawReversibleFrame(rec, SystemColors.Highlight, FrameStyle.Thick)

                    'Update control location according to mouse position
                    Dim mousePointToScreen As Point = Me.PointToScreen(New Point(e.X, e.Y))
                    Dim controlLocation As Point = Me.Location + (Me.Location - (Me.PointToScreen(Me.Location) - (mousePointToScreen - mousePositionInControl)))

                    'Draw control location
                    Me.Location = controlLocation
                    Me.Update()

                    'Draw outline
                    'rec = New Rectangle(controlLocation, Me.Size)
                    'rec.Inflate(2, 2)
                    'ControlPaint.DrawReversibleFrame(rec, SystemColors.Highlight, FrameStyle.Thick)

                Catch ex As Exception
                    MessageBox.Show(ex.ToString)

                End Try

            End If
        End If
    End Sub


    Private Sub ToolStrip1_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles ToolStrip1.MouseUp
        OnViewLocationChange(e)
        OnControlMoved(EventArgs.Empty)
        ControlDragOccured = False
    End Sub

    'Used for testing the drag operation
    Private Sub OnViewLocationChange(ByVal e As EventArgs)
        RaiseEvent ViewLocationChange(Me, e)
    End Sub

    'Fires if the control has moved
    Private Sub OnControlMoved(ByVal e As EventArgs)
        RaiseEvent ControlMoved(Me, e)
    End Sub 'OnControlMoved


    Public Sub OnGroupMenuItemClick(ByVal sender As Object, ByVal e As EventArgs)
        Dim menuItem As ToolStripMenuItem = CType(sender, ToolStripMenuItem)
        Dim args() As String = Split(menuItem.Name, "_")

        If menuItem.Checked Then
            menuItem.Checked = False
            'Get and remove child from parent
            Dim ch As PscChannel = CType(Me.DataSource, PeerChannel).RemovePeerChild(CInt(args(3)))
            OnUngroup(New PscChannelEventArgs(menuItem.Name, ch))
        Else
            menuItem.Checked = True
            OnGroup(New PscChannelEventArgs(menuItem.Name, Me.DataSource))
        End If

    End Sub

    Private Sub OnGroup(ByVal e As PscChannelEventArgs)
        RaiseEvent Group(Me, e)
        'UpdateGroupLabel()
    End Sub


    Private Sub OnUngroup(ByVal e As PscChannelEventArgs)
        RaiseEvent UnGroup(Me, e)
        'UpdateGroupLabel()
    End Sub


    Private Sub CenterToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CenterToolStripMenuItem.Click
        ICh.Position = 750
        ICh.move()
    End Sub

    Private Sub ICh_InverseChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ICh.InverseChanged
        OnChangedState(System.EventArgs.Empty)
    End Sub

    Private Sub OnChangedState(ByVal e As EventArgs)
        RaiseEvent StateChanged(Me, e)
    End Sub
End Class 'PscChannelControl




